import { Component, OnInit, Input, Output, EventEmitter, ViewContainerRef, ViewChild, OnDestroy, AfterViewInit } from '@angular/core';
import { AppEnvironment } from 'src/environments/AppEnvironment';
import { Subject, Subscription } from 'rxjs';
import { NzNotificationService } from 'ng-zorro-antd/notification';
import { Util } from '@global/util/util';
import { NzModalService } from 'ng-zorro-antd/modal';
import { Router, ActivatedRoute } from '@angular/router';
import { CodeList } from '@global/service/codelist/CodeList';
import { EditFormControl } from '@global/service/control/EditFormControl';


import { DEService } from '@global/service/de/DEService';
import { BXDService } from '@de/BXD/BXDService';
import { MainEditFormMode } from './main-form.mode';

import { FormButtonModel, FormPageModel, FormItemModel, FormDRUIPartModel, FormPartModel, FormGroupPanelModel, FormIFrameModel, FormRowItemModel, FormTabPageModel, FormTabPanelModel, FormUserControlModel } from '@global/model/form-detail';
import { debounceTime, distinctUntilChanged } from 'rxjs/operators';


@Component({
	selector: 'view_form',
	templateUrl: './main-form.html',
	styleUrls: ['./main-form.less'],
})
export class EditForm extends EditFormControl implements OnInit, OnDestroy, AfterViewInit {




    /**
     * 构造方法
     * @memberof Main
     */
    constructor(
        private $ctrlMode: MainEditFormMode,
        private $util: Util, private $modal: NzModalService,
        private $router: Router, private $route: ActivatedRoute, 
        private $notice: NzNotificationService,
        private $codelist: CodeList
        ,private $deService: BXDService
        
    ) {
        super();
	}

    /**
     * 获取部件模型
     *
     * @memberof Main
     */
    public getControlMode() {
        return this.$ctrlMode;
    }

    /**
     * 获取实体服务对象
     *
     * @returns
     * @memberof Main
     */
    public getDEService(): DEService{
        return this.$deService;
    }

    /**
     * 生命周期（页面加载完成）
     *
     * @memberof Grid
     */
    ngAfterViewInit(): void {
    }

    /**
     * 部件对象
     *
     * @type {*}
     * @memberof Main
     */
    @ViewChild('MainEditForm', {static: false}) public form: any;

    /**
     * 获取多项数据
     *
     * @returns {any[]}
     * @memberof Main
     */
    public getDatas(): any[] {
        return [this.data];
    }

    /**
     * 获取单项树
     *
     * @returns {*}
     * @memberof Main
     */
    public getData(): any {
        return this.data;
    }

    /**
     * Api地址
     *
     * @type {string}
     * @memberof Main
     */
    public url: string = 'cn.ibizlab.demo/test/bxd/maineditform/';

    /**
     * 表单数据对象
     *
     * @type {*}
     * @memberof Main
     */
    public data: any = {
        srfupdatedate: null,
        srforikey: null,
        srfkey: null,
        srfmajortext: null,
        srftempmode: null,
        srfuf: null,
        srfdeid: null,
        srfsourcekey: null,
        bxdname: null,
        createman: null,
        createdate: null,
        updateman: null,
        updatedate: null,
        bxdid: null,
    };

    /**
     * 详情模型集合
     *
     * @type {*}
     * @memberof Main
     */
    public detailsModel: any = {
        bxdname: new FormItemModel({ caption: '报销单名称', detailType: 'FORMITEM', name: 'bxdname', visible: true, isShowCaption: true, form: this, disabled: false, enableCond: 3 })
, 
        group1: new FormGroupPanelModel({ caption: '报销单基本信息', detailType: 'GROUPPANEL', name: 'group1', visible: true, isShowCaption: true, form: this, uiActionGroup: { caption: '', langbase: '', extractMode: 'ITEM', details: [] } })
, 
        formpage1: new FormPageModel({ caption: '基本信息', detailType: 'FORMPAGE', name: 'formpage1', visible: true, isShowCaption: true, form: this })
, 
        createman: new FormItemModel({ caption: '建立人', detailType: 'FORMITEM', name: 'createman', visible: true, isShowCaption: true, form: this, disabled: false, enableCond: 3 })
, 
        createdate: new FormItemModel({ caption: '建立时间', detailType: 'FORMITEM', name: 'createdate', visible: true, isShowCaption: true, form: this, disabled: false, enableCond: 3 })
, 
        updateman: new FormItemModel({ caption: '更新人', detailType: 'FORMITEM', name: 'updateman', visible: true, isShowCaption: true, form: this, disabled: false, enableCond: 3 })
, 
        updatedate: new FormItemModel({ caption: '更新时间', detailType: 'FORMITEM', name: 'updatedate', visible: true, isShowCaption: true, form: this, disabled: false, enableCond: 3 })
, 
        group2: new FormGroupPanelModel({ caption: '操作信息', detailType: 'GROUPPANEL', name: 'group2', visible: true, isShowCaption: true, form: this, uiActionGroup: { caption: '', langbase: '', extractMode: 'ITEM', details: [] } })
, 
        formpage2: new FormPageModel({ caption: '其它', detailType: 'FORMPAGE', name: 'formpage2', visible: true, isShowCaption: true, form: this })
, 
        form: new FormTabPanelModel({ caption: 'form', detailType: 'TABPANEL', name: 'form', visible: true, isShowCaption: true, form: this, tabPages: [{ name: 'formpage1', index: 0, visible: true }, { name: 'formpage2', index: 1, visible: true }] }),
    };

    /**
     * 属性值规则
     *
     * @type {*}
     * @memberof Main
     */
    public rules: any = {
        bxdname: {
            required: false,
            message: '报销单名称 值不能为空'
        },
        createman: {
            required: false,
            message: '建立人 值不能为空'
        },
        createdate: {
            required: false,
            message: '建立时间 值不能为空'
        },
        updateman: {
            required: false,
            message: '更新人 值不能为空'
        },
        updatedate: {
            required: false,
            message: '更新时间 值不能为空'
        },
    }

    /**
     * 获取表单模型
     *
     * @returns {*}
     * @memberof Main
     */
    public getFormDetailModel(): any {
        return this.detailsModel;
    }

    /**
     * 重置表单项值
     *
     * @param {{ name: string, newVal: any, oldVal: any }} { name, newVal, oldVal }
     * @memberof Main
     */
    public resetFormData({ name, newVal, oldVal }: { name: string, newVal: any, oldVal: any }): void {
    }

    /**
     * 表单逻辑
     *
     * @param {{ name: string, newVal: any, oldVal: any }} { name, newVal, oldVal }
     * @memberof Main
     */
    public formLogic({ name, newVal, oldVal }: { name: string, newVal: any, oldVal: any }): void {
                









    }

    /**
     * 表单加载完成
     *
     * @param {*} [data={}]
     * @memberof Main
     */
    public onFormLoad(data: any = {}): void {
        this.setFormEnableCond(data);
        this.fillForm(data);
        this.oldData = {};
        Object.assign(this.oldData, JSON.parse(JSON.stringify(this.data)));
        this.formLogic({ name: '', newVal: null, oldVal: null });
    }

    /**
     * 重置草稿表单状态
     *
     * @memberof Main
     */
    public resetDraftFormStates(): void {
        // const form: any = this.form;
        // if (form) {
        //     form.resetFields();
        // }
    }



    /**
     * 分组界面行为事件
     *
     * @param {*} $event
     * @memberof Main
     */
    public groupUIActionClick($event: any): void {
        if (!$event) {
            return;
        }
        const item:any = $event.item;
    }

    /**
     * 生命周期（销毁）
     *
     * @memberof Main
     */
    public ngOnDestroy(): void {
        if (this.dataChangEvent) {
            this.dataChangEvent.unsubscribe();
        }
    }

    /**
     * 加载
     *
     * @param {*} [opt={}]
     * @memberof Main
     */
    public load(opt: any = {}): void {
        const arg: any = { ...opt };

        this.getAction(arg).then((response: any) => {
            const data = response.data;
            this.onFormLoad(data);
            this.loadEvent.emit(data);
            this.formState.next({ type: 'load', data: data });
        }).catch((response: any) => {
            const { data: _data } = response;
            this.$notice.error(_data.title, _data.message);
        });
    }

    /**
     * 加载草稿
     *
     * @param {*} [opt={}]
     * @memberof Main
     */
    public loadDraft(opt: any = {}): void {
        const arg: any = { ...opt } ;

        this.getAction(arg).then((response: any) => {
            const data = response.data;
            this.resetDraftFormStates();
            this.onFormLoad(data);
            this.loadEvent.emit(data);
            this.formState.next({ type: 'load', data: data });
            setTimeout(() => {
                const form: any = this.form;
                if (form) {
                    form.fields.forEach((field: any) => {
                        field.validateMessage = "";
                        field.validateState = "";
                        field.validateStatus = false;
                    });
                }
            });
        }).catch((response: any) => {
            const { data: _data } = response;
            this.$notice.error(_data.title, _data.message);
        });
    }

    /**
     * 自动保存
     *
     * @param {*} [opt={}]
     * @memberof Main
     */
    public autoSave(opt: any = {}): void {
        const arg: any = { ...opt };
        const data = this.getValues();
        Object.assign(arg, data);

        this.updateAction(arg).then((response: any) => {
            const data = response.data;
            this.onFormLoad(data);
            this.saveEvent.emit(data);
            this.formState.next({ type: 'save', data: data });
        }).catch((response: any) => {
            const { data: _data } = response;
            this.$notice.error(_data.title, _data.message);
        });
    }

    /**
     * 保存
     *
     * @param {*} [opt={}]
     * @param {boolean} [showResultInfo]
     * @returns {Promise<any>}
     * @memberof Main
     */
    public async save(opt: any = {}, showResultInfo?: boolean): Promise<any> {
        showResultInfo = showResultInfo === undefined ? true : false;
        const arg: any = { ...opt };
        const data = this.getValues();
        Object.assign(arg, data);

        return new Promise((resolve: any, reject: any) => {
            this.updateAction(arg).then((response: any) => {
                const data = response.data;
                this.onFormLoad(data);
                this.saveEvent.emit(data);
                this.formState.next({ type: 'save', data: data });
                if (showResultInfo) {
                    this.$notice.success('', (data.srfmajortext ? data.srfmajortext : '') + '&nbsp;保存成功！');
                }

                resolve(response);
            }).catch((response: any) => {
                const { data: _data } = response;
                this.$notice.error(_data.title, _data.message);
                reject(response);
            });
        })
    }

    /**
     * 工作流提交
     *
     * @param {*} [data={}]
     * @returns {Promise<any>}
     * @memberof Main
     */
    public async wfstart(data: any = {}): Promise<any> {
        this.$notice.warning('', '暂不支持');
    }


    /**
     * 表单项更新
     *
     * @param {string} mode 界面行为名称
     * @param {*} [data={}] 请求数据
     * @param {string[]} updateDetails 更新项
     * @param {boolean} [showloading] 是否显示加载状态
     * @returns {void}
     * @memberof Main
     */
    public updateFormItems(mode: string, data: any = {}, updateDetails: string[], showloading?: boolean): void {
        this.$notice.warning('', '暂不支持');
    }

    /**
     * 回车事件
     *
     * @param {*} $event
     * @memberof Main
     */
    public onEnter($event: any): void {
    }

    /**
     * 生命周期
     *
     * @memberof SearchForm
     */
    ngOnInit(): void {
        if (this.viewState) {
            this.viewStateEvent = this.viewState.subscribe(({ tag, action, data }) => {
                if (!Object.is(tag, this.name)) {
                    return;
                }
                if (Object.is('autoload', action)) {
                    this.autoLoad(data);
                }
                if (Object.is('load', action)) {
                    this.load(data);
                }
                if (Object.is('loaddraft', action)) {
                    this.loadDraft(data);
                }
                if (Object.is('save', action)) {
                    this.save(data);
                }
            });
        }
    }

}
