import { Component, OnInit, Input, Output, EventEmitter, ViewContainerRef, ViewChild, OnDestroy, AfterViewInit } from '@angular/core';
import { AppEnvironment } from 'src/environments/AppEnvironment';
import { Subject, Subscription } from 'rxjs';
import { NzNotificationService } from 'ng-zorro-antd/notification';
import { Util } from '@global/util/util';
import { NzModalService } from 'ng-zorro-antd/modal';
import { Router, ActivatedRoute } from '@angular/router';
import { CodeList } from '@global/service/codelist/CodeList';
import { GridControl } from '@global/service/control/GridControl';


import { DEService } from '@global/service/de/DEService';
import { BXDService } from '@de/BXD/BXDService';
import { MainGridMode } from './main-grid.mode';



@Component({
	selector: 'view_grid',
	templateUrl: './main-grid.html',
	styleUrls: ['./main-grid.less'],
})
export class Grid extends GridControl implements OnInit, OnDestroy, AfterViewInit {




    /**
     * 构造方法
     * @memberof Main
     */
    constructor(
        private $ctrlMode: MainGridMode,
        private $util: Util, private $modal: NzModalService,
        private $router: Router, private $route: ActivatedRoute, 
        private $notice: NzNotificationService,
        private $codelist: CodeList
        ,private $deService: BXDService
        
    ) {
        super();
	}

    /**
     * 获取部件模型
     *
     * @memberof Main
     */
    public getControlMode() {
        return this.$ctrlMode;
    }

    /**
     * 获取实体服务对象
     *
     * @returns
     * @memberof Main
     */
    public getDEService(): DEService{
        return this.$deService;
    }

    /**
     * 生命周期（页面加载完成）
     *
     * @memberof Grid
     */
    ngAfterViewInit(): void {
    }

    /**
     * 部件对象
     *
     * @type {*}
     * @memberof Main
     */
    @ViewChild('MainGrid', {static: false}) public multipleTable: any;

    /**
     * 传入url值
     *
     * @type {string}
     * @memberof Main
     */
    private url: string = 'cn.ibizlab.demo/test/bxd/maingrid/';

    /**
     * 排序方向
     *
     * @type {string}
     * @memberof Main
     */
    public minorSortDir: string = '';

    /**
     * 排序字段
     *
     * @type {string}
     * @memberof Main
     */
    public minorSortPSDEF: string = '';

    /**
     * 是否显示标题
     *
     * @type {boolean}
     * @memberof Main
     */
    public isHideHeader: boolean = false;

    /**
     * 分页条数
     *
     * @type {number}
     * @memberof Main
     */
    public limit: number = 20;

    /**
     * 分页条数
     *
     * @returns {number}
     * @memberof Main
     */
    public getLimit(): number {
        return this.limit;
    }

    /**
     * 是否禁用排序
     *
     * @returns {boolean}
     * @memberof Main
     */
    public isNoSort(): boolean {
        return false;
    }

    /**
     * 是否启用分页
     *
     * @returns {boolean}
     * @memberof Main
     */
    public isEnablePagingBar(): boolean {
        return true;
    }

    /**
     * 删除
     *
     * @param {any[]} datas
     * @returns {Promise<any>}
     * @memberof Main
     */
    public async remove(datas: any[]): Promise<any> {
        if (datas.length === 0) {
            return;
        }
        let dataInfo = '';
        datas.forEach((record: any, index: number) => {
            let srfmajortext = record.srfmajortext;
            if (index < 5) {
                if (!Object.is(dataInfo, '')) {
                    dataInfo += '、';
                }
                dataInfo += srfmajortext;
            } else {
                return false;
            }
        });

        if (datas.length < 5) {
            dataInfo = dataInfo + '共' + datas.length + '条数据';
        } else {
            dataInfo = dataInfo + '...' + '共' + datas.length + '条数据';
        }

        const removeData = () => {
            let keys: any[] = [];
            datas.forEach((data: any) => {
                keys.push(data.srfkey);
            });

            return new Promise((resolve: any, reject: any) => {
                this.removeAction({ srfkeys: keys.join(';') }).then((response: any) => {

                    if (!response || response.status !== 200) {
                        this.$notice.error('', '删除数据失败,' + response.info);
                        return;
                    } else {
                        this.$notice.success('', '删除成功!');
                    }
                    this.load({});
                    this.removeEvent.emit(null);
                    this.selections = [];
                    resolve(response);
                }).catch((response: any) => {

                    if (response && response.status === 401) {
                        return;
                    }
                    if (!response || !response.status || !response.data) {
                        this.$notice.error('错误', '系统异常');
                        reject(response);
                        return;
                    }

                    const { data: _data } = response;
                    this.$notice.error( _data.title, _data.message);
                    reject(response);
                });
            });
        }

        dataInfo = dataInfo.replace(/[null]/g, '').replace(/[undefined]/g, '').replace(/[ ]/g, '');
        this.$modal.confirm({
            nzTitle: '警告',
            nzContent: '确认要删除 ' + dataInfo + '，删除操作将不可恢复？',
            nzOnOk: () => {
                removeData();
            },
            nzOnCancel: () => { }
        });
        return removeData;
    }


    /**
     * 批量添加
     *
     * @param {*} [arg={}]
     * @memberof Main
     */
    public addBatch(arg: any = {}): void {
        if (!arg) {
            arg = {};
        }
    }

    /**
     * 数据导出
     *
     * @param {*} data
     * @memberof Main
     */
    public exportExcel(data: any = {}): void {
        const arg: any = {};

        const page: any = {};
        if (!this.isNoSort && !Object.is(this.minorSortDir, '') && !Object.is(this.minorSortPSDEF, '')) {
            const sort: string[] = [this.minorSortPSDEF];
            if (Object.is(this.minorSortDir, 'ASC')) {
                Object.assign(page, { asc: sort });
            } else {
                Object.assign(page, { desc: sort });
            }
        }

        if (Object.is(data.type, 'maxRowCount')) {
            Object.assign(page, { current: 1, size: data.maxRowCount });
        } else if (Object.is(data.type, 'activatedPage')) {
            Object.assign(page, { current: this.curPage, size: this.limit });
        }
        Object.assign(arg, { page: page });

        const parentdata: any = {};
        this.beforeloadEvent.emit(parentdata);
        Object.assign(arg, parentdata);

    }

    /**
     * 生命周期
     *
     * @memberof SearchForm
     */
    ngOnInit(): void {
        if (this.viewState) {
            this.viewStateEvent = this.viewState.subscribe(({ tag, action, data }) => {
                if (!Object.is(tag, this.name)) {
                    return;
                }
                if (Object.is('load', action)) {
                    this.load(data);
                }
                if (Object.is('remove', action)) {
                    this.load(data);
                }
            });
        }
    }

    /**
     * 分页条数变化
     *
     * @param {*} $event
     * @returns {void}
     * @memberof GridService
     */
    public onPageSizeChange($event: any): void {
        if (!$event) {
            return;
        }
        if ($event === this.limit){
            return;
        }
        this.limit = $event;
        this.curPage = 1;
        this.load({});
    }

    /**
     * 页面变化
     *
     * @param {*} $event
     * @returns {void}
     * @memberof GridService
     */
    public pageOnChange($event: any): void {
        if (!$event) {
            return;
        }
        if ($event === this.curPage) {
            return;
        }
        this.curPage = $event;
        this.load({});
    }

    /**
     * 排序变化
     *
     * @param {*} order
     * @param {*} prop
     * @returns {void}
     * @memberof Main
     */
    public onSortChange(order: any, prop: any): void {
        const dir = Object.is(order, 'ascend') ? 'ASC' : Object.is(order, 'descend') ? 'DESC' : '';
        if (Object.is(dir, this.minorSortDir) && Object.is(this.minorSortPSDEF, prop)) {
            return;
        }
        this.minorSortDir = dir;
        this.minorSortPSDEF = prop ? prop : '';
        this.load({});
    }

    /**
     * 界面行为
     *
     * @param {*} row
     * @param {*} tag
     * @param {*} $event
     * @memberof Main
     */
	public uiAction(row: any, tag: any, $event: any) {
        this.rowClick(row);
    }

}
